#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "GL/glew.h"
#include "GL/wglew.h"
#include "glm/glm.hpp"
#include "glm/ext.hpp"

#include "shader.h"
#include "model.h"
#include "RenderTarget.h"
#include "Camera.h"
#include "SkyBox.h"

#include "AL/alut.h"

#include <QMainWindow>
#include <QTimer>
#include <QElapsedTimer>
#include <QKeyEvent>
#include <QMouseEvent>

QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

    void InitGL(QWidget *pWidget);
    void Init();
public slots:
    void Draw();

private:
    bool eventFilter(QObject *obj, QEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void keyReleaseEvent(QKeyEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);

    Ui::MainWindow *ui;

    // OpenGL
    HWND hWnd;
    HDC hDC;
    HGLRC hRC;

    // .OBJ file
    Model m_model;

    // SkyBox
    SkyBox m_SkyBox;

    QTimer m_timer;

    // FPS
    QElapsedTimer m_elapsedTimer;
    uint64_t m_nElapsedTime;
    uint64_t m_nCurrentTime;
    float dt;
    int nFPS = 0;
    float fSec = 0;

    // shader
    Shader m_shaderDraw;
    Shader m_shaderShadowMap;

    // Camera
    Camera m_camera;
    bool m_bMouseButtonDown = false;
    bool bKeys[256];
    void SetKeys(int id, bool bPressed);

    // RenderTarget
    int m_nShadowWidth = 2048;
    RenderTarget m_RenderToShadowTexture;

    bool m_bIsDrawWireframe = false;
};
#endif // MAINWINDOW_H
