#ifndef MODEL_H
#define MODEL_H

#include <vector>
#include <string>

#include "GL/glew.h"
#include "GL/wglew.h"
#include "glm/glm.hpp"
#include "glm/ext.hpp"

#include "texture.h"
#include "shader.h"

typedef struct _Vertex
{
    glm::vec3 v3Position;
    glm::vec3 v3Normal;
    glm::vec2 v2TextCoord;
}
Vertex;

typedef struct _Material
{
    std::vector< Vertex > m_listVertices;

    Texture *m_pAlbedoTexture = nullptr;
    Texture *m_pDispTexture = nullptr;

    std::vector< uint32_t > m_listIndices;
    GLuint m_glIndexBuffer = 0;
}
Material;

class Model
{
public:
    explicit Model();
    ~Model();

    void Load(std::string strDir, std::string strFilename, glm::mat4 matTransform = glm::mat4(1.0f), bool bIsLoadTextures = true);
    void Draw(Shader *shader, uint32_t mode);
    void Release();
    void SetDispFactor(float fValue);
    float GetDispFactor();

    void CreateOpenGLBuffers();
    std::vector< Vertex >* GetVertices();
    std::vector< Material >* GetMaterials();

private:
    void ApplyTessellation(float fArea);
    void Tessellation(Vertex A, Vertex B, Vertex C, float fArea, QList< Vertex > &tessellationVertices);
    void GetCenter(Vertex &ret, Vertex A, Vertex B);
    float GetTriangleArea(glm::vec3 A, glm::vec3 B, glm::vec3 C);

    std::vector< Material > m_listMaterials;

    GLuint m_glVertexBuffer = 0;
    std::vector< Vertex > m_listAllVertices;

    bool m_bIsLoadTextures;

    float m_fDispFactor = 0.0f;
};

#endif // MODEL_H
